//---------------------------------------------------------------------------
// Copyright 2004 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHmath/DataConv.c,v $
// $Id: DataConv.c,v 1.2 2004/04/06 23:14:32 ah Exp $ 
//
// Description: Implementation of class DataConv
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: DataConv.c,v 1.2 2004/04/06 23:14:32 ah Exp $ (c) 2004 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
#include <AH/Math/DataConv.h>
USING_NAMESPACE(AH)

#include <stdlib.h>
#include <math.h>
#include <string>
#include <vector>
USING_NAMESPACE(std)

#include <AH/Error/Intern.h>
USING_NAMESPACE(AH)

BEGIN_NAMESPACE(AH)

// --- local definitions ---

#define PROFILING 0
#define TRACE 0

// --- local constants ---

// --- local variables ---

// --- local classes ---

// --- local functions ---

// --- member functions ---

void DataConv::reduceLinear
(
    const double* data, unsigned int dataNum,
    unsigned int outNum,
    double* outMean, double* outMax, double* outMin
)
{
    // do some checks:
    if (!outMean && !outMax && !outMin)
	return; // nothing to do at all
    if (0 == dataNum) 
	throw ErrorInternBadArg("dataNum = 0 in DataConv::reduceLinear()");
    if (0 == outNum)
	throw ErrorInternBadArg("outNum = 0 in DataConv::reduceLinear()");
    if (!data)
	throw ErrorInternNullPtrArg("data in DataConv::reduceLinear()");

    // handle simple case
    if (outNum == dataNum)
    {
	// just keep current data
	for (unsigned int i = 0; i < dataNum; ++i)
	{
	    if (outMean) outMean[i] = data[i];
	    if (outMax)  outMax[i] = data[i];
	    if (outMin)  outMin[i] = data[i];
	}
	return;
    }

    // interpolate or extrapolate data
    const double d_inc = (double)dataNum / (double)outNum;
    const double dlim_inc = 0.5 * d_inc;
    double d = 0;
    for (unsigned int p = 0; p < outNum; ++p, d += d_inc)
    {
	// determine index span (float) of this plot point:
	double di1 = d - dlim_inc;
	double di2 = d + dlim_inc;

	// determine first and last index:
	int bi1 = (int)(di1 + 0.5);
	int bi2 = (int)(di2 + 0.5);
	if (bi1 < 0) bi1 = 0;
	if ((unsigned int)bi2 >= dataNum) bi2 = dataNum -1;

	// determine proportional parts of first and last index:
	double pi1; // proportional part of bi1
	double pi2; // proportional part of bi2
	if (bi1 == bi2)
	{
	    pi1 = 0.5 * fabs(di2 - di1);
	    pi2 = pi1;
	}
	else
	{
	    pi1 = (double)bi1 + 0.5 - di1;
	    pi2 = fabs(di2 - (double)bi2 + 0.5);
	}

	// find mean, max and min value:
	double max = data[bi1];
	if (max < data[bi2]) max = data[bi2];
	double min = data[bi1];
	if (min > data[bi2]) min = data[bi2];

	double sum = pi1 * data[bi1] + pi2 * data[bi2];
	double cnt = pi1 + pi2;
	for (int i = bi1+1; i < bi2; ++i)
	{
	    sum += data[i];
	    if (min > data[i]) min = data[i];
	    if (max < data[i]) max = data[i];
	    ++cnt;
	}
	if (outMean) outMean[p] = sum / cnt; // mean value
	if (outMax)  outMax[p]  = max;
	if (outMin)  outMin[p]  = min;
    }
}


END_NAMESPACE
